@@ -7,9 +7,9 @@
  * 
  * JasPer License Version 2.0
  * 
+ * Copyright (c) 2001-2006 Michael David Adams
  * Copyright (c) 1999-2000 Image Power, Inc.
  * Copyright (c) 1999-2000 The University of British Columbia
- * Copyright (c) 2001-2003 Michael David Adams
  * 
  * All rights reserved.
  * 
@@ -71,6 +71,7 @@
 #include "jasper/jas_stream.h"
 #include "jasper/jas_image.h"
 #include "jasper/jas_string.h"
+#include "jasper/jas_debug.h"
 
 #include "jpg_jpeglib.h"
 #include "jpg_cod.h"
@@ -144,37 +145,64 @@ jas_image_t *jpg_decode(jas_stream_t *in, char *optstr)
 	/* Avoid compiler warnings about unused parameters. */
 	optstr = 0;
 
+	// In theory, the two memset calls that follow are not needed.
+	// They are only here to make the code more predictable in the event
+	// that the JPEG library fails to initialize a member.
+	memset(&cinfo, 0, sizeof(struct jpeg_decompress_struct));
+	memset(dest_mgr, 0, sizeof(jpg_dest_t));
+
+	dest_mgr->data = 0;
+
 	image = 0;
 	input_file = 0;
 	if (!(input_file = tmpfile())) {
+		jas_eprintf("cannot make temporary file\n");
 		goto error;
 	}
 	if (jpg_copystreamtofile(input_file, in)) {
+		jas_eprintf("cannot copy stream\n");
 		goto error;
 	}
 	rewind(input_file);
 
 	/* Allocate and initialize a JPEG decompression object. */
+	JAS_DBGLOG(10, ("jpeg_std_error(%p)\n", &jerr));
 	cinfo.err = jpeg_std_error(&jerr);
+	JAS_DBGLOG(10, ("jpeg_create_decompress(%p)\n", &cinfo));
 	jpeg_create_decompress(&cinfo);
 
 	/* Specify the data source for decompression. */
+	JAS_DBGLOG(10, ("jpeg_stdio_src(%p, %p)\n", &cinfo, input_file));
 	jpeg_stdio_src(&cinfo, input_file);
 
 	/* Read the file header to obtain the image information. */
+	JAS_DBGLOG(10, ("jpeg_read_header(%p, TRUE)\n", &cinfo));
 	jpeg_read_header(&cinfo, TRUE);
+	JAS_DBGLOG(10, (
+	  "header: image_width %d; image_height %d; num_components %d\n",
+	  cinfo.image_width, cinfo.image_height, cinfo.num_components)
+	  );
 
 	/* Start the decompressor. */
+	JAS_DBGLOG(10, ("jpeg_start_decompress(%p)\n", &cinfo));
 	jpeg_start_decompress(&cinfo);
+	JAS_DBGLOG(10, (
+	  "header: output_width %d; output_height %d; output_components %d\n",
+	  cinfo.output_width, cinfo.output_height, cinfo.output_components)
+	  );
 
 	/* Create an image object to hold the decoded data. */
 	if (!(image = jpg_mkimage(&cinfo))) {
+		jas_eprintf("jpg_mkimage failed\n");
 		goto error;
 	}
 
 	/* Initialize the data sink object. */
 	dest_mgr->image = image;
-	dest_mgr->data = jas_matrix_create(1, cinfo.output_width);
+	if (!(dest_mgr->data = jas_matrix_create(1, cinfo.output_width))) {
+		jas_eprintf("jas_matrix_create failed\n");
+		goto error;
+	}
 	dest_mgr->start_output = jpg_start_output;
 	dest_mgr->put_pixel_rows = jpg_put_pixel_rows;
 	dest_mgr->finish_output = jpg_finish_output;
@@ -187,6 +215,8 @@ jas_image_t *jpg_decode(jas_stream_t *in, char *optstr)
 	/* Process the compressed data. */
 	(*dest_mgr->start_output)(&cinfo, dest_mgr);
 	while (cinfo.output_scanline < cinfo.output_height) {
+		JAS_DBGLOG(10, ("jpeg_read_scanlines(%p, %p, %ul)\n", &cinfo,
+		  dest_mgr->buffer, JAS_CAST(unsigned long, dest_mgr->buffer_height)));
 		num_scanlines = jpeg_read_scanlines(&cinfo, dest_mgr->buffer,
 		  dest_mgr->buffer_height);
 		(*dest_mgr->put_pixel_rows)(&cinfo, dest_mgr, num_scanlines);
@@ -194,22 +224,30 @@ jas_image_t *jpg_decode(jas_stream_t *in, char *optstr)
 	(*dest_mgr->finish_output)(&cinfo, dest_mgr);
 
 	/* Complete the decompression process. */
+	JAS_DBGLOG(10, ("jpeg_finish_decompress(%p)\n", &cinfo));
 	jpeg_finish_decompress(&cinfo);
 
 	/* Destroy the JPEG decompression object. */
+	JAS_DBGLOG(10, ("jpeg_destroy_decompress(%p)\n", &cinfo));
 	jpeg_destroy_decompress(&cinfo);
 
 	jas_matrix_destroy(dest_mgr->data);
 
+	JAS_DBGLOG(10, ("fclose(%p)\n", input_file));
 	fclose(input_file);
+	input_file = 0;
 
 	if (dest_mgr->error) {
+		jas_eprintf("error during decoding\n");
 		goto error;
 	}
 
 	return image;
 
 error:
+	if (dest_mgr->data) {
+		jas_matrix_destroy(dest_mgr->data);
+	}
 	if (image) {
 		jas_image_destroy(image);
 	}
@@ -236,6 +274,12 @@ static jas_image_t *jpg_mkimage(j_decompress_ptr cinfo)
 		goto error;
 	}
 	for (cmptno = 0; cmptno < numcmpts; ++cmptno) {
+		if (cinfo->image_width < JAS_IMAGE_COORD_MIN ||
+		  cinfo->image_width > JAS_IMAGE_COORD_MAX ||
+		  cinfo->image_height < JAS_IMAGE_COORD_MIN ||
+		  cinfo->image_height > JAS_IMAGE_COORD_MAX) {
+			goto error;
+		}
 		cmptparm.tlx = 0;
 		cmptparm.tly = 0;
 		cmptparm.hstep = 1;
@@ -300,6 +344,8 @@ static void jpg_start_output(j_decompress_ptr cinfo, jpg_dest_t *dinfo)
 	/* Avoid compiler warnings about unused parameters. */
 	cinfo = 0;
 
+	JAS_DBGLOG(10, ("jpg_start_output(%p, %p)\n", cinfo, dinfo));
+
 	dinfo->row = 0;
 }
 
@@ -311,6 +357,8 @@ static void jpg_put_pixel_rows(j_decompress_ptr cinfo, jpg_dest_t *dinfo,
 	JDIMENSION x;
 	uint_fast32_t width;
 
+	JAS_DBGLOG(10, ("jpg_put_pixel_rows(%p, %p)\n", cinfo, dinfo));
+
 	if (dinfo->error) {
 		return;
 	}
@@ -334,6 +382,8 @@ static void jpg_put_pixel_rows(j_decompress_ptr cinfo, jpg_dest_t *dinfo,
 
 static void jpg_finish_output(j_decompress_ptr cinfo, jpg_dest_t *dinfo)
 {
+	JAS_DBGLOG(10, ("jpg_finish_output(%p, %p)\n", cinfo, dinfo));
+
 	/* Avoid compiler warnings about unused parameters. */
 	cinfo = 0;
 	dinfo = 0;
