@@ -70,6 +70,7 @@
 #include "jasper/jas_image.h"
 #include "jasper/jas_string.h"
 #include "jasper/jas_malloc.h"
+#include "jasper/jas_debug.h"
 
 #include "mif_cod.h"
 
@@ -175,6 +176,7 @@ jas_image_t *mif_decode(jas_stream_t *in, char *optstr)
 		cmpt = hdr->cmpts[cmptno];
 		tmpstream = cmpt->data ? jas_stream_fopen(cmpt->data, "rb") : in;
 		if (!tmpstream) {
+			jas_eprintf("cannot open component file %s\n", cmpt->data);
 			goto error;
 		}
 		if (!(tmpimage = jas_image_decode(tmpstream, -1, 0))) {
@@ -309,14 +311,18 @@ int mif_encode(jas_image_t *image, jas_stream_t *out, char *optstr)
 			cmptparm.height = cmpt->height;
 			cmptparm.prec = cmpt->prec;
 			cmptparm.sgnd = false;
-			if (jas_image_addcmpt(tmpimage, jas_image_numcmpts(tmpimage), &cmptparm)) {
+			if (jas_image_addcmpt(tmpimage, jas_image_numcmpts(tmpimage),
+			  &cmptparm)) {
 				goto error;
 			}
+			jas_image_setclrspc(tmpimage, JAS_CLRSPC_SGRAY);
+			jas_image_setcmpttype(tmpimage, 0,
+			  JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_GRAY_Y));
 			if (!(data = jas_seq2d_create(0, 0, cmpt->width, cmpt->height))) {
 				goto error;
 			}
-			if (jas_image_readcmpt(image, cmptno, 0, 0, cmpt->width, cmpt->height,
-			  data)) {
+			if (jas_image_readcmpt(image, cmptno, 0, 0, cmpt->width,
+			  cmpt->height, data)) {
 				goto error;
 			}
 			if (cmpt->sgnd) {
@@ -327,8 +333,8 @@ int mif_encode(jas_image_t *image, jas_stream_t *out, char *optstr)
 					}
 				}
 			}
-			if (jas_image_writecmpt(tmpimage, 0, 0, 0, cmpt->width, cmpt->height,
-			  data)) {
+			if (jas_image_writecmpt(tmpimage, 0, 0, 0, cmpt->width,
+			  cmpt->height, data)) {
 				goto error;
 			}
 			jas_seq2d_destroy(data);
@@ -391,7 +397,10 @@ int mif_validate(jas_stream_t *in)
 	}
 
 	/* Compute the signature value. */
-	magic = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
+	magic = (JAS_CAST(uint_fast32_t, buf[0]) << 24) |
+	  (JAS_CAST(uint_fast32_t, buf[1]) << 16) |
+	  (JAS_CAST(uint_fast32_t, buf[2]) << 8) |
+	  buf[3];
 
 	/* Ensure that the signature is correct for this format. */
 	if (magic != MIF_MAGIC) {
@@ -438,8 +447,8 @@ static int mif_hdr_growcmpts(mif_hdr_t *hdr, int maxcmpts)
 	int cmptno;
 	mif_cmpt_t **newcmpts;
 	assert(maxcmpts >= hdr->numcmpts);
-	newcmpts = (!hdr->cmpts) ? jas_malloc(maxcmpts * sizeof(mif_cmpt_t *)) :
-	  jas_realloc(hdr->cmpts, maxcmpts * sizeof(mif_cmpt_t *));
+	newcmpts = (!hdr->cmpts) ? jas_alloc2(maxcmpts, sizeof(mif_cmpt_t *)) :
+	  jas_realloc2(hdr->cmpts, maxcmpts, sizeof(mif_cmpt_t *));
 	if (!newcmpts) {
 		return -1;
 	}
@@ -461,6 +470,7 @@ static mif_hdr_t *mif_hdr_get(jas_stream_t *in)
 	int id;
 
 	hdr = 0;
+	tvp = 0;
 
 	if (jas_stream_read(in, magicbuf, MIF_MAGICLEN) != MIF_MAGICLEN) {
 		goto error;
@@ -479,26 +489,39 @@ static mif_hdr_t *mif_hdr_get(jas_stream_t *in)
 	done = false;
 	do {
 		if (!mif_getline(in, buf, sizeof(buf))) {
+			jas_eprintf("mif_getline failed\n");
 			goto error;
 		}
 		if (buf[0] == '\0') {
 			continue;
 		}
+		JAS_DBGLOG(10, ("header line: len=%d; %s\n", strlen(buf), buf));
 		if (!(tvp = jas_tvparser_create(buf))) {
+			jas_eprintf("jas_tvparser_create failed\n");
 			goto error;
 		}
 		if (jas_tvparser_next(tvp)) {
-			abort();
+			jas_eprintf("cannot get record type\n");
+			goto error;
 		}
-		id = jas_taginfo_nonull(jas_taginfos_lookup(mif_tags2, jas_tvparser_gettag(tvp)))->id;
+		id = jas_taginfo_nonull(jas_taginfos_lookup(mif_tags2,
+		  jas_tvparser_gettag(tvp)))->id;
 		jas_tvparser_destroy(tvp);
+		tvp = 0;
 		switch (id) {
 		case MIF_CMPT:
-			mif_process_cmpt(hdr, buf);
+			if (mif_process_cmpt(hdr, buf)) {
+				jas_eprintf("cannot get component information\n");
+				goto error;
+			}
 			break;
 		case MIF_END:
 			done = 1;
 			break;
+		default:
+			jas_eprintf("invalid header information: %s\n", buf);
+			goto error;
+			break;
 		}
 	} while (!done);
 
@@ -508,6 +531,9 @@ error:
 	if (hdr) {
 		mif_hdr_destroy(hdr);
 	}
+	if (tvp) {
+		jas_tvparser_destroy(tvp);
+	}
 	return 0;
 }
 
@@ -521,6 +547,7 @@ static int mif_process_cmpt(mif_hdr_t *hdr, char *buf)
 	tvp = 0;
 
 	if (!(cmpt = mif_cmpt_create())) {
+		jas_eprintf("cannot create component\n");
 		goto error;
 	}
 	cmpt->tlx = 0;
@@ -534,8 +561,17 @@ static int mif_process_cmpt(mif_hdr_t *hdr, char *buf)
 	cmpt->data = 0;
 
 	if (!(tvp = jas_tvparser_create(buf))) {
+		jas_eprintf("cannot create parser\n");
 		goto error;
 	}
+
+	// Skip the component keyword
+	if ((id = jas_tvparser_next(tvp))) {
+		// This should never happen.
+		abort();
+	}
+
+	// Process the tag-value pairs.
 	while (!(id = jas_tvparser_next(tvp))) {
 		switch (jas_taginfo_nonull(jas_taginfos_lookup(mif_tags,
 		  jas_tvparser_gettag(tvp)))->id) {
@@ -565,15 +601,23 @@ static int mif_process_cmpt(mif_hdr_t *hdr, char *buf)
 			break;
 		case MIF_DATA:
 			if (!(cmpt->data = jas_strdup(jas_tvparser_getval(tvp)))) {
-				return -1;
+				goto error;
 			}
 			break;
+		default:
+			jas_eprintf("invalid component information: %s\n", buf);
+			goto error;
+			break;
 		}
 	}
 	if (!cmpt->sampperx || !cmpt->samppery) {
 		goto error;
 	}
+	if (!cmpt->width || !cmpt->height || !cmpt->prec || cmpt->sgnd < 0) {
+		goto error;
+	}
 	if (mif_hdr_addcmpt(hdr, hdr->numcmpts, cmpt)) {
+		jas_eprintf("cannot add component\n");
 		goto error;
 	}
 	jas_tvparser_destroy(tvp);
@@ -692,15 +736,16 @@ static int mif_getc(jas_stream_t *in)
 	do {
 		switch (c = jas_stream_getc(in)) {
 		case EOF:
-			done = 1;
+			done = true;
 			break;
 		case '#':
 			for (;;) {
 				if ((c = jas_stream_getc(in)) == EOF) {
-					done = 1;
+					done = true;
 					break;
 				}	
 				if (c == '\n') {
+					done = true;
 					break;
 				}
 			}
@@ -711,7 +756,7 @@ static int mif_getc(jas_stream_t *in)
 			}
 			break;
 		default:
-			done = 1;
+			done = true;
 			break;
 		}
 	} while (!done);
@@ -735,7 +780,9 @@ static mif_hdr_t *mif_makehdrfromimage(jas_image_t *image)
 	hdr->magic = MIF_MAGIC;
 	hdr->numcmpts = jas_image_numcmpts(image);
 	for (cmptno = 0; cmptno < hdr->numcmpts; ++cmptno) {
-		hdr->cmpts[cmptno] = jas_malloc(sizeof(mif_cmpt_t));
+		if (!(hdr->cmpts[cmptno] = jas_malloc(sizeof(mif_cmpt_t)))) {
+			goto error;
+		}
 		cmpt = hdr->cmpts[cmptno];
 		cmpt->tlx = jas_image_cmpttlx(image, cmptno);
 		cmpt->tly = jas_image_cmpttly(image, cmptno);
@@ -748,4 +795,15 @@ static mif_hdr_t *mif_makehdrfromimage(jas_image_t *image)
 		cmpt->data = 0;
 	}
 	return hdr;
+
+error:
+	for (cmptno = 0; cmptno < hdr->numcmpts; ++cmptno) {
+		if (hdr->cmpts[cmptno]) {
+			jas_free(hdr->cmpts[cmptno]);
+		}
+	}
+	if (hdr) {
+		jas_free(hdr);
+	}
+	return 0;
 }
